function deepClone(obj) {
  if (obj === null || typeof obj !== 'object') {
    return obj;
  }
  let clone = Array.isArray(obj) ? [] : {};
  for (let key in obj) {
    clone[key] = deepClone(obj[key]);
  }
  return clone;
}

function showLocation(folderPath) {
  return folderPath.join('/');
}

function createFolder(gallery, parentFolderPath, folderName) {
  const newGallery = deepClone(gallery);
  let target = newGallery;
  for (const folder of parentFolderPath) {
    target = target[folder].items;
  }
  if (target[folderName]) {
    throw new Error('Folder already exists');
  }
  target[folderName] = { type: 'folder', items: {} };
  return newGallery;
}

function createImage(gallery, parentFolderPath, imageName, src) {
  const newGallery = deepClone(gallery);
  let target = newGallery;
  for (const folder of parentFolderPath) {
    target = target[folder].items;
  }
  target[imageName] = { src };
  return newGallery;
}

function moveItems(gallery= {}, fromFolderPath= [], toFolderPath= [], items= []) {
  
  if (toFolderPath.length ) {
    let parentFolder = toFolderPath[toFolderPath.length - 1];
    if(items.includes(parentFolder)){
      throw new Error('Cannot move a folder into itself');
    };
  }

  const newGallery = deepClone(gallery);

  let fromFolder = newGallery;
  for (const folder of fromFolderPath) {
    if(fromFolderPath.length && fromFolder[folder].type!== 'folder'){
      throw new Error('Bad from folder path');
    }
    fromFolder = fromFolder[folder].items || {};
  };
  
  let toFolder = newGallery;
  for (const folder of toFolderPath) {
    if(toFolderPath.length && toFolder[folder].type!== 'folder'){
      throw new Error('Bad to folder path');
    }
    toFolder = toFolder[folder].items || {};
  };

  items.forEach(item => {
    // avoid overriting existing items w/ same key
    if(toFolder[item]) {
      // while loop to find a unique key
      let i = 1;
      while(toFolder[item + i]) {
        i++;
      }
      toFolder[item + i] = fromFolder[item];
    }else{
      toFolder[item] = fromFolder[item];
    };
    
    delete fromFolder[item];
  });

  return newGallery;
}

function deleteItems(gallery, folderPath, items) {
  const newGallery = deepClone(gallery);
  let target = newGallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }

  items.forEach(item => {
    delete target[item];
  });

  return newGallery;
}

function openFolder(gallery, folderPath) {
  let target = gallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }
  return deepClone(target);
}

function updateItem(gallery, folderPath, itemKey, newItem) {
  const newGallery = deepClone(gallery);
  let target = newGallery;
  for (const folder of folderPath) {
    target = target[folder].items;
  }
  target[itemKey] = newItem;
  return newGallery;
}

const getObject = (gallery, folderPath, id) => {
  let folder = deepClone(gallery);
  for (const key of folderPath) {
    if (folder[key] && folder[key].type === 'folder') {
      folder = folder[key].items;
    } else {
      return null; // Path is invalid
    }
  }
  return folder[id] || null; // Return the item or null if not found
};

export {
	deepClone,
	showLocation,
	createFolder,
	createImage,
	moveItems,
	deleteItems,
	openFolder,
	updateItem,
  getObject
}