import { createFolder } from '../src'; // Adjust the import path as needed

describe('createFolder', () => {
  it('should create a new folder in the gallery', () => {
    const initialGallery = {
      parentFolder: { type: 'folder', items: {} },
    }; 
    const newGallery = createFolder(initialGallery, ['parentFolder'], 'newFolder');
    expect(newGallery.parentFolder.items).toHaveProperty(['newFolder']);
    expect(newGallery.parentFolder.items.newFolder).toEqual({ type: 'folder', items: {} });
  });

  it('should throw an error if the folder already exists', () => {
    const initialGallery = {
      parentFolder: { type: 'folder', items: {} },
    }; 
    expect(() => createFolder(initialGallery, [], 'parentFolder')).toThrow();
  });

  it('should throw an error if the parent folder does not exist', () => {
    const initialGallery = {
      parentFolder: { type: 'folder', items: {} },
    }; 
    expect(() => createFolder(initialGallery, ['nonexistentFolder'], 'newFolder')).toThrow();
  });

  it('should throw an error if the parent folder is not a folder', () => {
    const initialGallery = {
      parentFolder: { type: 'image', src: 'https://example.com/image.jpg' },
    }; 
    expect(() => createFolder(initialGallery, ['parentFolder'], 'newFolder')).toThrow();
  });

});
